const int blinkPin = 13;
const int ledPins[] = { 7, 6, 5, 4 };
const int buttonPins[] = { 11, 10, 9, 8 };

enum Mode {
  OFF, DISCO, BUTTONS, BUTTONS_INVERTED
} mode = DISCO;

#define NOTE_B0  31
#define NOTE_C1  33
#define NOTE_CS1 35
#define NOTE_D1  37
#define NOTE_DS1 39
#define NOTE_E1  41
#define NOTE_F1  44
#define NOTE_FS1 46
#define NOTE_G1  49
#define NOTE_GS1 52
#define NOTE_A1  55
#define NOTE_AS1 58
#define NOTE_B1  62
#define NOTE_C2  65
#define NOTE_CS2 69
#define NOTE_D2  73
#define NOTE_DS2 78
#define NOTE_E2  82
#define NOTE_F2  87
#define NOTE_FS2 93
#define NOTE_G2  98
#define NOTE_GS2 104
#define NOTE_A2  110
#define NOTE_AS2 117
#define NOTE_B2  123
#define NOTE_C3  131
#define NOTE_CS3 139
#define NOTE_D3  147
#define NOTE_DS3 156
#define NOTE_E3  165
#define NOTE_F3  175
#define NOTE_FS3 185
#define NOTE_G3  196
#define NOTE_GS3 208
#define NOTE_A3  220
#define NOTE_AS3 233
#define NOTE_B3  247
#define NOTE_C4  262
#define NOTE_CS4 277
#define NOTE_D4  294
#define NOTE_DS4 311
#define NOTE_E4  330
#define NOTE_F4  349
#define NOTE_FS4 370
#define NOTE_G4  392
#define NOTE_GS4 415
#define NOTE_A4  440
#define NOTE_AS4 466
#define NOTE_B4  494
#define NOTE_C5  523
#define NOTE_CS5 554
#define NOTE_D5  587
#define NOTE_DS5 622
#define NOTE_E5  659
#define NOTE_F5  698
#define NOTE_FS5 740
#define NOTE_G5  784
#define NOTE_GS5 831
#define NOTE_A5  880
#define NOTE_AS5 932
#define NOTE_B5  988
#define NOTE_C6  1047
#define NOTE_CS6 1109
#define NOTE_D6  1175
#define NOTE_DS6 1245
#define NOTE_E6  1319
#define NOTE_F6  1397
#define NOTE_FS6 1480
#define NOTE_G6  1568
#define NOTE_GS6 1661
#define NOTE_A6  1760
#define NOTE_AS6 1865
#define NOTE_B6  1976
#define NOTE_C7  2093
#define NOTE_CS7 2217
#define NOTE_D7  2349
#define NOTE_DS7 2489
#define NOTE_E7  2637
#define NOTE_F7  2794
#define NOTE_FS7 2960
#define NOTE_G7  3136
#define NOTE_GS7 3322
#define NOTE_A7  3520
#define NOTE_AS7 3729
#define NOTE_B7  3951
#define NOTE_C8  4186
#define NOTE_CS8 4435
#define NOTE_D8  4699
#define NOTE_DS8 4978

void setup() {
  Serial.begin(1200);
  
  pinMode(blinkPin, OUTPUT);
  for(int i = 0; i < 4; i ++){
    pinMode(ledPins[i], OUTPUT);
    //pinMode(buttonPins[i], INPUT_PULLUP);
  }

  pinMode(8, OUTPUT);
  
  pinMode(3, INPUT);
  pinMode(2, OUTPUT);
}

void setState(int8_t state) {

  int frequencies[] = { NOTE_C5, NOTE_D5, NOTE_E5, NOTE_F5 };
  //int frequencies[] = { 500, 1000, 2000, 4000 };

  if (state == 0) noTone(8);
  else if (state <= 4) tone(8, frequencies[state - 1]);
  else noTone(8);
  
  for (int i = 0; i < 4; ++i) { 
    digitalWrite(ledPins[i], ledPins[i] == 8 - state);
  }
}

void ProcessCommand(const char* cmd){

  if (strcmp(cmd, "blink_on") == 0) digitalWrite(blinkPin, true);
  else if (strcmp(cmd, "blink_off") == 0) digitalWrite(blinkPin, false);
  else if (strcmp(cmd, "disco") == 0) mode = DISCO;
  else if (strcmp(cmd, "off") == 0) mode = OFF;
  else if (strcmp(cmd, "buttons") == 0) mode = BUTTONS;
  else if (strcmp(cmd, "buttons_inverted") == 0) mode = BUTTONS_INVERTED;
  else if (strcmp(cmd, "notes") == 0) {

    char notes[300] = {0};
    char noteCount = 0;
    long intervals[300] = {0};
    char intervalCount = 0;

    //Read the notes
    while(true){
      while(!Serial.available());
      char c = Serial.read();
      //Serial.write(c);
      if (c == '\n') break;
      else notes[noteCount++] = c;
      //Serial.print("Note ");
      //Serial.print(noteCount - 1);
      //Serial.print(": ");
      //Serial.println(notes[noteCount-1]);
    }

    //Read the intervals
    while(true){
      while(!Serial.available());
      char c = Serial.read();
      //Serial.write(c);
      if (c == '\n') break;
      else {
        char* data = (char*)&intervals[intervalCount++];
        data[0] = c;
        while(!Serial.available());
        data[1] = Serial.read();
        while(!Serial.available());
        data[2] = Serial.read();
        while(!Serial.available());
        data[3] = Serial.read();
      }
      //Serial.print("Interval ");
      //Serial.print(intervalCount - 1);
      //Serial.print(": ");
      //Serial.println(intervals[intervalCount-1]);
    }

    //Check for bugs
    if (intervalCount != noteCount){
      Serial.println("Interval count does not match note count!");
      return;
    }

    //Play!
    for(int i = 0; i < noteCount; i ++){
      setState(notes[i]);
      unsigned long startTime = millis();
      while (millis() - startTime < intervals[i]);
    }
    setState(0);
  }
}

void UpdateMode(){

  if (mode == OFF){
    for(int i = 0; i < 4; i ++){
      digitalWrite(ledPins[i], false);
    }
  }
  else if (mode == DISCO){
    for(int i = 0; i < 4; i ++){
      digitalWrite(ledPins[i], random(0, 2));
    }
  }
  else if (mode == BUTTONS){
    for(int i = 0; i < 4; i ++){
      digitalWrite(ledPins[i], !digitalRead(buttonPins[i]));
    }
  }
  else if (mode == BUTTONS_INVERTED){
    for(int i = 0; i < 4; i ++){
      digitalWrite(ledPins[i], digitalRead(buttonPins[i]));
    }
  }
}

char cmdBuffer[32];
int cmdBufferIndex = 0;
void loop() {

  UpdateMode();
  delay(100);

  while (Serial.available()){
    char c = Serial.read();
    Serial.write(c);

    if (c == '\n'){
      cmdBuffer[cmdBufferIndex] = 0;
      ProcessCommand(cmdBuffer);
      cmdBufferIndex = 0;
    }
    else{
      cmdBuffer[cmdBufferIndex++] = c;
    }
  }
}
